var rimwheel_enabled;
var rimwheel_rimsize;
var rimwheel_pageratio;
var rimwheel_scrollsize;
var rimwheel_debug;

rimwheel_load_settings();

var rimwheel_x = 0;
var rimwheel_y = 0;

window.addEventListener("mousemove", rimwheel_move_handler, false);
window.addEventListener("DOMMouseScroll", rimwheel_wheel_handler, false);

function rimwheel_load_settings()
{
	var prefs = Components.classes["@mozilla.org/preferences-service;1"]
		.getService(Components.interfaces.nsIPrefService)
		.getBranch("extensions.rimwheel.");

	rimwheel_enabled = rimwheel_getBoolPref(prefs, "enabled", true);
	rimwheel_rimsize = rimwheel_getIntPref(prefs, "rimsize", 64);
	rimwheel_pageratio = rimwheel_getCharPref(prefs, "pageratio", 0.9);
	rimwheel_scrollsize = rimwheel_getIntPref(prefs, "scrollsize", 48);
	rimwheel_debug = rimwheel_getBoolPref(prefs, "debug", false);
}

function rimwheel_getBoolPref(prefs, name, d)
{
	if (prefs.prefHasUserValue(name))
		return prefs.getBoolPref(name);
	else
		return d;
}

function rimwheel_getIntPref(prefs, name, d)
{
	if (prefs.prefHasUserValue(name))
		return prefs.getIntPref(name);
	else
		return d;
}

function rimwheel_getCharPref(prefs, name, d)
{
	if (prefs.prefHasUserValue(name))
		return prefs.getCharPref(name);
	else
		return d;
}

RimwheelPrefsObserver.prototype =
{
	observe:
		function(subject, topic, data) {
			rimwheel_load_settings();
		}
}

var rimwheel_prefs_observer = new RimwheelPrefsObserver();

function RimwheelPrefsObserver()
{
	try {
		var prefs = Components.classes["@mozilla.org/preferences-service;1"]
			.getService(Components.interfaces.nsIPrefService)
			.getBranch(null);
		var pbi = prefs.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
		pbi.addObserver("extensions.rimwheel", this, false);
	} catch (e) {
		alert(e);
	}
}

function rimwheel_move_handler(e)
{
	rimwheel_x = e.screenX - gBrowser.boxObject.screenX;
	rimwheel_y = e.screenY - gBrowser.boxObject.screenY;
}

function rimwheel_wheel_handler(e)
{
	if (rimwheel_debug) {
		alert("(x,y)=("+e.screenX+","+e.screenY+"), "
			+"(x,y)=("+rimwheel_x+","+rimwheel_y+"), "
			+"target="+e.target+", "
			+"localName="+e.target.localName+", "
			+"ownerDocument="+e.target.ownerDocument);
	}

	if (!rimwheel_enabled) return;

	var ix, iy;

	if (rimwheel_x < rimwheel_rimsize) ix = 0;
	else if (rimwheel_x < gBrowser.boxObject.width - rimwheel_rimsize) ix = 1;
	else ix = 2;

	if (rimwheel_y < rimwheel_rimsize) iy = 0;
	else if (rimwheel_y < gBrowser.boxObject.height - rimwheel_rimsize) iy = 1;
	else iy = 2;

	switch (iy * 3 + ix) {

		case 0: // upper left
			if (e.detail < 0) // up
				BrowserStop();
			else // down
				BrowserReload();
			e.preventDefault();
			return;

		case 1: // upper
			if (e.detail < 0) // up
				gBrowser.tabContainer.advanceSelectedTab(e.detail > 0 ? 1 : -1, true);
			else // down
				gBrowser.tabContainer.advanceSelectedTab(e.detail > 0 ? 1 : -1, true);
			e.preventDefault();
			return;

		case 2: // upper right
			if (e.detail < 0) // up
				BrowserCloseTabOrWindow();
			else // down
				delayedOpenTab();
			e.preventDefault();
			return;

		case 3: // left
			if (e.detail < 0) // up
				BrowserBack();
			else // down
				BrowserForward();
			e.preventDefault();
			return;

		case 4: // center
			return; // do default operation

		case 5: // right
			if (e.detail < 0) // up
				window._content.scrollBy(0, -window._content.innerHeight * rimwheel_pageratio);
			else // down
				window._content.scrollBy(0, window._content.innerHeight * rimwheel_pageratio);
			e.preventDefault();
			return;

		case 6: // lower left
			if (e.detail < 0) // up
				BrowserCloseWindow();
			else // down
				BrowserHome();
			e.preventDefault();
			return;

		case 7: // lower
			if (e.detail < 0) // up
				e.target.ownerDocument.defaultView.scrollBy(-rimwheel_scrollsize, 0);
			else // down
				e.target.ownerDocument.defaultView.scrollBy(rimwheel_scrollsize, 0);
			e.preventDefault();
			return;

		case 8: // lower right
			if (e.detail < 0) // up
				window._content.scrollTo(window._content.scrollX, 0);
			else // down
				window._content.scrollTo(window._content.scrollX, window._content.scrollMaxY);
			e.preventDefault();
			return;
	}
}
